"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.highlight = void 0;
const path_1 = __importDefault(require("path"));
const compiler_1 = require("svelte/compiler");
const magic_string_1 = __importDefault(require("magic-string"));
const highlight_js_1 = __importDefault(require("highlight.js"));
const prettier_1 = require("prettier");
require("prettier-plugin-svelte");
const languages = new Set([...highlight_js_1.default.listLanguages(), "svelte", "html"]);
const highlight = (options) => {
    const ignorePath = (options === null || options === void 0 ? void 0 : options.ignorePath) || ((filename) => /(node_modules|.svelte-kit)/.test(filename));
    const prettierOptions = options === null || options === void 0 ? void 0 : options.prettierOptions;
    return {
        markup({ content, filename }) {
            if (!filename)
                return;
            if (ignorePath(filename))
                return;
            if (!content)
                return;
            const s = new magic_string_1.default(content);
            const log = (...message) => {
                const file = path_1.default.relative(process.cwd(), filename);
                console.log(`[${file}]`, ...message);
            };
            (0, compiler_1.walk)((0, compiler_1.parse)(content), {
                enter(node) {
                    var _a, _b, _c;
                    if (node.name === "pre") {
                        let code = "";
                        let language;
                        const { attributes, children } = node;
                        const expression = (_a = children.find((child) => (child === null || child === void 0 ? void 0 : child.type) === "MustacheTag")) === null || _a === void 0 ? void 0 : _a.expression;
                        if (expression === null || expression === void 0 ? void 0 : expression.value) {
                            /**
                             * @example
                             * <pre data-language="typescript">
                             *   {'const a = (b: number) => 4'}
                             * </pre>
                             */
                            code = expression.value;
                        }
                        else if (expression === null || expression === void 0 ? void 0 : expression.quasis) {
                            /**
                             * @example
                             * <pre data-language="typescript">
                             *   {\`const a = (b: number) => 4\`}
                             * </pre>
                             */
                            code = (_b = expression.quasis[0]) === null || _b === void 0 ? void 0 : _b.value.raw;
                        }
                        language = (_c = attributes.find(({ name }) => name === "data-language")) === null || _c === void 0 ? void 0 : _c.value[0].raw;
                        /**
                         * Assume normal usage of the `<pre>` element
                         * if no `data-language="<language>"` is provided.
                         */
                        if (!language)
                            return;
                        if (!code)
                            return log("No code provided");
                        if (!languages.has(language))
                            return log(`Invalid language "${language}"`);
                        let formatted = code;
                        try {
                            formatted = (0, prettier_1.format)(code, Object.assign({ parser: language }, prettierOptions));
                        }
                        catch (error) {
                            if (error && typeof error === "object") {
                                log("Formatting error", error.codeFrame);
                            }
                        }
                        const { value: highlighted } = /svelte|html/.test(language)
                            ? highlight_js_1.default.highlightAuto(formatted, ["xml", "css", "javascript"])
                            : highlight_js_1.default.highlight(formatted, { language });
                        s.overwrite(node.start, node.end, `<pre><code class="hljs">{@html \`${highlighted}\`}</code></pre>`);
                    }
                },
            });
            return {
                code: s.toString(),
                map: s.generateMap({ file: filename, includeContent: true }),
            };
        },
    };
};
exports.highlight = highlight;
